@@ -1,6 +1,6 @@
 /*
  *******************************************************************************
- * Copyright (C) 2002-2008, International Business Machines Corporation and    *
+ * Copyright (C) 2002-2009, International Business Machines Corporation and    *
  * others. All Rights Reserved.                                                *
  *******************************************************************************
  *
@@ -200,7 +200,8 @@ public class TestConversion extends ModuleTest {
         try {
             // if cc.charset starts with '*', obtain it from com/ibm/icu/dev/data/testdata
             charset = (cc.charset != null && cc.charset.length() > 0 && cc.charset.charAt(0) == '*')
-                    ? (Charset) provider.charsetForName(cc.charset.substring(1), "../dev/data/testdata")
+                    ? (Charset) provider.charsetForName(cc.charset.substring(1),
+                        "com/ibm/icu/dev/data/testdata", this.getClass().getClassLoader())
                     : (Charset) provider.charsetForName(cc.charset);
             encoder = (CharsetEncoder) charset.newEncoder();
             encoder.onMalformedInput(CodingErrorAction.REPLACE);
@@ -213,8 +214,7 @@ public class TestConversion extends ModuleTest {
             }
             
         } catch (Exception e) {
-            // TODO implement loading of test data.
-            if (skipIfBeforeICU(4,1,0)) {
+            if (skipIfBeforeICU(4,3,0)) { // TIME BOMB
                 logln("Skipping test:(" + cc.charset + ") due to ICU Charset not supported at this time");
             } else {
                 errln(cc.charset + " was not found");
@@ -222,9 +222,6 @@ public class TestConversion extends ModuleTest {
             return;
         }
         
-        
-        
-        
         // set the callback for the encoder 
         if (cc.cbErrorAction != null) {
             if (cc.cbEncoder != null) {
@@ -490,7 +487,8 @@ public class TestConversion extends ModuleTest {
         try {
             // if cc.charset starts with '*', obtain it from com/ibm/icu/dev/data/testdata
             charset = (cc.charset != null && cc.charset.length() > 0 && cc.charset.charAt(0) == '*')
-                    ? (Charset) provider.charsetForName(cc.charset.substring(1), "../dev/data/testdata")
+                    ? (Charset) provider.charsetForName(cc.charset.substring(1),
+                        "com/ibm/icu/dev/data/testdata", this.getClass().getClassLoader())
                     : (Charset) provider.charsetForName(cc.charset);
             decoder = (CharsetDecoder) charset.newDecoder();
             decoder.onMalformedInput(CodingErrorAction.REPLACE);
@@ -498,7 +496,7 @@ public class TestConversion extends ModuleTest {
 
         } catch (Exception e) {
             // TODO implement loading of test data.
-            if (skipIfBeforeICU(4,1,0)) {
+            if (skipIfBeforeICU(4,3,0)) {
                 logln("Skipping test:(" + cc.charset + ") due to ICU Charset not supported at this time");
             } else {
                 errln(cc.charset + " was not found");
@@ -869,8 +867,9 @@ public class TestConversion extends ModuleTest {
         try{
            // if cc.charset starts with '*', obtain it from com/ibm/icu/dev/data/testdata
            charset = (cc.charset != null && cc.charset.length() > 0 && cc.charset.charAt(0) == '*')
-                   ? (CharsetICU) provider.charsetForName(cc.charset.substring(1), "../dev/data/testdata")
-                   : (CharsetICU) provider.charsetForName(cc.charset);
+                    ? (CharsetICU) provider.charsetForName(cc.charset.substring(1),
+                        "com/ibm/icu/dev/data/testdata", this.getClass().getClassLoader())
+                    : (CharsetICU) provider.charsetForName(cc.charset);
            
            //checking for converter that are not supported at this point        
            try{
@@ -1095,38 +1094,20 @@ public class TestConversion extends ModuleTest {
         output.limit(output.position());
         output.rewind();
 
-//TODO: Fix Me!  After Ticket#6583 is completed, this code should be removed.
-        boolean ignoreError = (0 <= cc.caseNr && cc.caseNr <= 15) || cc.caseNr == 17 || cc.caseNr == 18;
-//TODO: End
-
         // test to see if the conversion matches actual results
         if (output.limit() != expected.length()) {
-//TODO: Remove this
-            if (ignoreError) {
-                logln("Test failed: output length does not match expected for charset: "+cc.charset+ " [" + cc.caseNr + "]");
+            if (skipIfBeforeICU(4,3,0)) { // TIME BOMB
+                logln("Skipping test:(" + cc.charset + ") due to time bomb");
             } else {
                 errln("Test failed: output length does not match expected for charset: "+cc.charset+ " [" + cc.caseNr + "]");
-                res = false;
             }
-//TODO: End
-//            errln("Test failed: output length does not match expected for charset: "+cc.charset+ " [" + cc.caseNr + "]");
-//            res = false;
+            res = false;
         } else {
             for (int i = 0; i < expected.length(); i++) {
                 if (output.get(i) != expected.charAt(i)) {
-//TODO: Remove this
-                    if (ignoreError) {
-                        logln("Test failed: output does not match expected for charset: " + cc.charset
-                                + " [" + cc.caseNr + "]");
-                    } else {
-                        errln("Test failed: output does not match expected for charset: " + cc.charset
-                                + " [" + cc.caseNr + "]");
-                        res = false;
-                    }
-//TODO: End
-//                    errln("Test failed: output does not match expected for charset: " + cc.charset
-//                            + " [" + cc.caseNr + "]");
-//                    res = false;
+                    errln("Test failed: output does not match expected for charset: " + cc.charset
+                            + " [" + cc.caseNr + "]");
+                    res = false;
                     break;
                 }
             }
@@ -1139,7 +1120,9 @@ public class TestConversion extends ModuleTest {
             logln("Expected:    " + printchars(CharBuffer.wrap(expected), expected.length()));
             logln("Passed");
         }
-        else {
+        else if (skipIfBeforeICU(4,3,0)) {
+            // TIME BOMB
+        } else {
             errln("[" + cc.caseNr + "]:" + cc.charset);
             errln("Input:       " + printbytes(cc.bytes, cc.bytes.limit()));
             errln("Output:      " + printchars(output, output.limit()));
