@@ -1,6 +1,6 @@
 /*
  *******************************************************************************
- * Copyright (C) 2002-2005, International Business Machines Corporation and    *
+ * Copyright (C) 2002-2007, International Business Machines Corporation and    *
  * others. All Rights Reserved.                                                *
  *******************************************************************************
  */
@@ -1016,7 +1016,7 @@ public class CollationRegressionTest extends TestFmwk {
         // where the same token (after normalization) appears twice in a row
         boolean caughtException = false;
         try {
-            RuleBasedCollator dummy = new RuleBasedCollator("\u00e0<a\u0300");
+            new RuleBasedCollator("\u00e0<a\u0300");
         }
         catch (ParseException e) {
             caughtException = true;
@@ -1042,7 +1042,7 @@ public class CollationRegressionTest extends TestFmwk {
     // CollationElementIterator.previous broken for expanding char sequences
     //
     public void Test4179686() throws Exception {
-
+        RuleBasedCollator en_us = (RuleBasedCollator) Collator.getInstance(Locale.US);
         // Create a collator with a few expanding character sequences in it....
         RuleBasedCollator coll = new RuleBasedCollator(en_us.getRules()
                                                     + " & ae ; \u00e4 & AE ; \u00c4"
@@ -1074,12 +1074,9 @@ public class CollationRegressionTest extends TestFmwk {
         }
     }
 
-    private static RuleBasedCollator en_us ;
     protected void init()throws Exception{
-        if(en_us ==null){
-            en_us = (RuleBasedCollator)Collator.getInstance(Locale.US);
-        }
     }
+    
     public void Test4244884() throws Exception {
         RuleBasedCollator coll = (RuleBasedCollator)Collator.getInstance(Locale.US);
         coll = new RuleBasedCollator(coll.getRules()
