@@ -120,25 +120,6 @@ public class TimeZoneFormatTest extends com.ibm.icu.dev.test.TestFmwk {
                                     bFailure = !(canonicalID.indexOf('/') == -1)
                                                 && !((BasicTimeZone)outtz).hasEquivalentTransitions(tz, low, high);
                                 }
-                                if (bFailure) {
-                                    // There are tzids not supported by CLDR.  For example,
-                                    // new tzids introduced in the tz database after CLDR release.
-                                    // In 4.0, we used to use GMT as fallback, but we changed the
-                                    // fallback behavior in later releases.
-                                    // This workaround contains list of tzids known to fail.
-                                    // There is not good way to identify such zones (no corresponding
-                                    // CLDR data) except for accessing the resource data directly.
-                                    final String[] excludedTZIDs = {
-                                        "America/Argentina/Salta", "America/Santarem",
-                                        "Asia/Riyadh87", "Asia/Riyadh88", "Asia/Riyadh89",
-                                    };
-                                    for (int i = 0; i < excludedTZIDs.length; i++) {
-                                        if (canonicalID.equals(excludedTZIDs[i])) {
-                                            bFailure = false;
-                                            break;
-                                        }
-                                    }
-                                }
                                 if (bFailure) {
                                     errln("Canonical round trip failed; tz=" + tzids[tzidx]
                                             + ", locale=" + LOCALES[locidx] + ", pattern=" + PATTERNS[patidx]
@@ -205,8 +186,7 @@ public class TimeZoneFormatTest extends com.ibm.icu.dev.test.TestFmwk {
      */
     public void TestTimeRoundTrip() {
 
-        //boolean TEST_ALL = "true".equalsIgnoreCase(getProperty("TimeZoneRoundTripAll"));
-        boolean TEST_ALL = false;
+        boolean TEST_ALL = "true".equalsIgnoreCase(getProperty("TimeZoneRoundTripAll"));
 
         int startYear, endYear;
 
@@ -288,8 +268,6 @@ public class TimeZoneFormatTest extends com.ibm.icu.dev.test.TestFmwk {
                     long t = START_TIME;
                     TimeZoneTransition tzt = null;
                     boolean middle = true;
-                    boolean isJdkTimeZone = (TimeZone.getDefaultTimeZoneType() == TimeZone.TIMEZONE_JDK);
-testTransitions:
                     while (t < END_TIME) {
                         if (tzt == null) {
                             testTimes[0] = t;
@@ -349,24 +327,10 @@ testTransitions:
                             }
                             times[patidx] += System.currentTimeMillis() - timer;
                         }
-
-                        long tmpt = t;
-                        while (tmpt < END_TIME) {
-                            tzt = btz.getNextTransition(tmpt, false);
-                            if (tzt == null) {
-                                break testTransitions;
-                            }
-                            // JDK time zone data may be out of synch
-                            if (!isJdkTimeZone || (tz.getOffset(tzt.getTime() - 1) != tz.getOffset(tzt.getTime()))) {
-                                break;
-                            }
-                            tmpt = tzt.getTime();
-                        }
                         tzt = btz.getNextTransition(t, false);
                         if (tzt == null) {
-                            break testTransitions;
+                            break;
                         }
-
                         if (middle) {
                             // Test the date in the middle of two transitions.
                             t += (tzt.getTime() - t)/2;
