@@ -1,9 +1,9 @@
-//##header J2SE15
+//##header
 //#if defined(FOUNDATION10) || defined(J2SE13)
 //#else
 /*
  *******************************************************************************
- * Copyright (C) 2002-2007, International Business Machines Corporation and    *
+ * Copyright (C) 2002-2009, International Business Machines Corporation and    *
  * others. All Rights Reserved.                                                *
  *******************************************************************************
 */
@@ -46,7 +46,6 @@ public class TimeZoneAliasTest extends TestFmwk {
      * 2. all aliases must have the same offsets
       */
     public void TestAliases() {
-        if (skipIfBeforeICU(3,0,0)) return;
         Zone.Seconds seconds = new Zone.Seconds();
         for (Iterator it = Zone.getZoneSet().iterator(); it.hasNext(); ) {
             Zone zone = (Zone)it.next();
@@ -87,7 +86,6 @@ public class TimeZoneAliasTest extends TestFmwk {
      * We check to see that every timezone that is not an alias is actually different!
      */
     public void TestDifferences() {
-        if (skipIfBeforeICU(3,0,0)) return;
         Zone last = null;
         Zone.Seconds diffDate = new Zone.Seconds();        
         for (Iterator it = Zone.getZoneSet().iterator(); it.hasNext();) {
