@@ -1,7 +1,7 @@
-//##header J2SE15
+//##header
 /**
  *******************************************************************************
- * Copyright (C) 2000-2009, International Business Machines Corporation and    *
+ * Copyright (C) 2000-2010, International Business Machines Corporation and    *
  * others. All Rights Reserved.                                                *
  *******************************************************************************
  */
@@ -1071,16 +1071,10 @@ public class TimeZoneRegression extends TestFmwk {
                 errln("FAIL: Modified zone(" + tz.getID() + ") - getRawOffset returns " + offset + "/ Expected: " + newRawOffset);
             }
             // Ticket#5917
-            // Check if DST observation status is not unexpectedly changed to true.
-            // When original Olson time zone observes DST, setRawOffset may change DST observation
-            // status for some zones.  For example, Asia/Jerusalem, which currently use no DST after
-            // 2037 in tzdata 2007g.  But, the opposite change (false -> true) should never happen.
-            if (!useDst) {
-                if (tz.useDaylightTime()) {
-                    // Note: we have no plan to merge the fix for #6818 to ICU 4.0 maintenance stream
-                    //errln("FAIL: Modified zone(" + tz.getID() + ") - useDaylightTime has changed from false to true.");
-                    logln("FAIL: Modified zone(" + tz.getID() + ") - useDaylightTime has changed from false to true.");
-                }
+            // Check if DST observation status is not unexpectedly changed.
+            boolean newDst = tz.useDaylightTime();
+            if (useDst != newDst) {
+                errln("FAIL: Modified zone(" + tz.getID() + ") - useDaylightTime has changed from " + useDst + " to " + newDst);
             }
             // Make sure the offset is preserved in a clone
             TimeZone tzClone = (TimeZone)tz.clone();
@@ -1149,7 +1143,33 @@ public class TimeZoneRegression extends TestFmwk {
         } else {
             logln("OK: zone " + j_id +" returned offset in December: " + offset2);
         }
+    }
 
+    public void TestT7107() {
+        Thread[] workers = new Thread[20];
+        for (int i = 0 ; i < workers.length; i++) {
+            workers[i] = new Thread(new Runnable() {
+                public void run() {
+                    for (int j = 0; j < 10000; j++) {
+                        try {
+                            com.ibm.icu.util.TimeZone.getTimeZone("GMT").getDisplayName();
+                        } catch (Exception e) {
+                            errln("FAIL: Caught an exception " + e);
+                        }
+                    }
+                }
+            });
+        }
+        for (int i = 0; i < workers.length; i++) {
+            workers[i].start();
+        }
+        for (int i = 0; i < workers.length; i++) {
+            try {
+                workers[i].join();
+            } catch (InterruptedException ie) {
+                
+            }
+        }
     }
 }
 
