@@ -1,9 +1,9 @@
-//##header J2SE15
+//##header
 //#if defined(FOUNDATION10) || defined(J2SE13)
 //#else
 /*
  *******************************************************************************
- * Copyright (C) 2002-2008, International Business Machines Corporation and    *
+ * Copyright (C) 2002-2009, International Business Machines Corporation and    *
  * others. All Rights Reserved.                                                *
  *******************************************************************************
  */
@@ -17,6 +17,8 @@ import java.util.Iterator;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
+import java.util.Set;
+import java.util.TreeSet;
 
 import com.ibm.icu.lang.UCharacter;
 import com.ibm.icu.lang.UProperty;
@@ -184,8 +186,7 @@ public class ICUPropertyFactory extends UnicodeProperty.Factory {
             public List _getAvailableValues(List result) {
                 if (result == null) result = new ArrayList();
                 if (propEnum == UProperty.AGE) {
-                    addAllUnique(new String[] {
-                        "unassigned","1.1","2.0","2.1","3.0","3.1","3.2","4.0"},
+                    addAllUnique(getAges(),
                         result);
                     return result;
                 }
@@ -218,6 +219,19 @@ public class ICUPropertyFactory extends UnicodeProperty.Factory {
                 return result;
             }
 
+            static String[] AGES = null;
+            private String[] getAges() {
+                if (AGES == null) {
+                  Set ages = new TreeSet();
+                  for (int i = 0; i < 0x10FFFF; ++i) {
+                    VersionInfo age = UCharacter.getAge(i);
+                    ages.add(age.toString());
+                  }
+                  AGES = (String[]) ages.toArray(new String[ages.size()]);
+                }
+                return AGES;
+            }
+
             public List _getValueAliases(String valueAlias, List result) {
                 if (result == null) result = new ArrayList();
                 if (propEnum == UProperty.AGE) {
