@@ -1,6 +1,6 @@
 /*
  *******************************************************************************
- * Copyright (C) 1996-2008, International Business Machines Corporation and    *
+ * Copyright (C) 1996-2009, International Business Machines Corporation and    *
  * others. All Rights Reserved.                                                *
  *******************************************************************************
  */
@@ -29,7 +29,8 @@ public class TestAll extends TestGroup {
             "LocaleDataTest",
             "ULocaleTest",
             "LocaleAliasTest",
-            "DebugUtilitiesTest"
+            "DebugUtilitiesTest",
+            "LocaleBuilderTest",
         },
               "Test miscellaneous public utilities");
     }
