@@ -1,7 +1,7 @@
-//##header J2SE15
+//##header
 /*
  *******************************************************************************
- * Copyright (C) 1996-2008, International Business Machines Corporation and    *
+ * Copyright (C) 1996-2009, International Business Machines Corporation and    *
  * others. All Rights Reserved.                                                *
  *******************************************************************************
  */
@@ -14,6 +14,7 @@ import com.ibm.icu.impl.UCharacterProperty;
 import com.ibm.icu.impl.Utility;
 import com.ibm.icu.util.ULocale;
 import com.ibm.icu.util.UResourceBundle;
+import com.ibm.icu.util.UResourceBundleIterator;
 
 import java.math.BigInteger;
 import java.text.FieldPosition;
@@ -508,6 +509,13 @@ public class RuleBasedNumberFormat extends NumberFormat {
      */
     public static final int DURATION = 3;
 
+    /**
+     * Selector code that tells the constructor to create a numbering system formatter
+     * @draft ICU 4.2
+     * @provisional This API might change or be removed in a future release.
+     */
+    public static final int NUMBERING_SYSTEM = 4;
+
     //-----------------------------------------------------------------------
     // data members
     //-----------------------------------------------------------------------
@@ -588,6 +596,11 @@ public class RuleBasedNumberFormat extends NumberFormat {
     
     private static final boolean DEBUG  =  ICUDebug.enabled("rbnf");
 
+    // Temporary workaround - when noParse is true, do noting in parse.
+    // TODO: We need a real fix - see #6895/#6896
+    private boolean noParse;
+    private static final String[] NO_SPELLOUT_PARSE_LANGUAGES = { "ga" };
+
     //-----------------------------------------------------------------------
     // constructors
     //-----------------------------------------------------------------------
@@ -718,10 +731,12 @@ public class RuleBasedNumberFormat extends NumberFormat {
      * and duration.
      * @param locale The locale for the formatter.
      * @param format A selector code specifying which kind of formatter to create for that
-     * locale.  There are three legal values: SPELLOUT, which creates a formatter that
+     * locale.  There are four legal values: SPELLOUT, which creates a formatter that
      * spells out a value in words in the desired language, ORDINAL, which attaches
      * an ordinal suffix from the desired language to the end of a number (e.g. "123rd"),
-     * and DURATION, which formats a duration in seconds as hours, minutes, and seconds.
+     * DURATION, which formats a duration in seconds as hours, minutes, and seconds, and
+     * NUMBERING_SYSTEM, which is used to invoke rules for alternate numbering
+     * systems such as the Hebrew numbering system, or for Roman numerals, etc..
      * @stable ICU 3.2
      */
     public RuleBasedNumberFormat(ULocale locale, int format) {
@@ -740,7 +755,22 @@ public class RuleBasedNumberFormat extends NumberFormat {
         String[][] localizations = null;
 
         try {
+            // For backwards compatability - If we have a pre-4.2 style RBNF resource, attempt to read it.
             description = bundle.getString(rulenames[format-1]);
+        }
+        catch (MissingResourceException e) {
+            try {
+                ICUResourceBundle rules = bundle.getWithFallback("RBNFRules/"+rulenames[format-1]);
+                UResourceBundleIterator it = rules.getIterator(); 
+                while (it.hasNext()) {
+                   description = description.concat(it.nextString());
+                }
+            }
+            catch (MissingResourceException e1) {
+            } 
+        }
+
+        try {
             UResourceBundle locb = bundle.get(locnames[format-1]);
             localizations = new String[locb.getSize()][];
             for (int i = 0; i < localizations.length; ++i) {
@@ -752,13 +782,25 @@ public class RuleBasedNumberFormat extends NumberFormat {
         }
 
         init(description, localizations);
+
+        //TODO: we need a real fix - see #6895 / #6896
+        noParse = false;
+        if (locnames[format-1].equals("SpelloutLocalizations")) {
+            String lang = locale.getLanguage();
+            for (int i = 0; i < NO_SPELLOUT_PARSE_LANGUAGES.length; i++) {
+                if (NO_SPELLOUT_PARSE_LANGUAGES[i].equals(lang)) {
+                    noParse = true;
+                    break;
+                }
+            }
+        }
     }
 
     private static final String[] rulenames = {
-        "SpelloutRules", "OrdinalRules", "DurationRules",
+        "SpelloutRules", "OrdinalRules", "DurationRules", "NumberingSystemRules",
     };
     private static final String[] locnames = {
-        "SpelloutLocalizations", "OrdinalLocalizations", "DurationLocalizations",
+        "SpelloutLocalizations", "OrdinalLocalizations", "DurationLocalizations", "NumberingSystemLocalizations",
     };
 
     /**
@@ -769,6 +811,7 @@ public class RuleBasedNumberFormat extends NumberFormat {
      * out a value in words in the default locale's langyage, ORDINAL, which attaches
      * an ordinal suffix from the default locale's language to a numeral, and
      * DURATION, which formats a duration in seconds as hours, minutes, and seconds.
+     * or NUMBERING_SYSTEM, which is used for alternate numbering systems such as Hebrew.
      * @stable ICU 2.0
      */
     public RuleBasedNumberFormat(int format) {
@@ -1134,6 +1177,12 @@ public class RuleBasedNumberFormat extends NumberFormat {
      */
     public Number parse(String text, ParsePosition parsePosition) {
 
+        //TODO: We need a real fix.  See #6895 / #6896
+        if (noParse) {
+            // skip parsing
+            return new Long(0);
+        }
+
         // parsePosition tells us where to start parsing.  We copy the
         // text in the string from here to the end inro a new string,
         // and create a new ParsePosition and result variable to use
@@ -1153,7 +1202,7 @@ public class RuleBasedNumberFormat extends NumberFormat {
         // the result we return
         for (int i = ruleSets.length - 1; i >= 0; i--) {
             // skip private rule sets
-            if (ruleSets[i].getName().startsWith("%%")) {
+            if (!ruleSets[i].isPublic() || !ruleSets[i].isParseable()) {
                 continue;
             }
 
@@ -1256,14 +1305,23 @@ public class RuleBasedNumberFormat extends NumberFormat {
             if (publicRuleSetNames.length > 0) {
                 defaultRuleSet = findRuleSet(publicRuleSetNames[0]);
             } else {
-            defaultRuleSet = null;
-            int n = ruleSets.length;
-        while (--n >= 0) {
-          if (ruleSets[n].isPublic()) {
-            defaultRuleSet = ruleSets[n];
-            break;
-          }
-        }
+                defaultRuleSet = null;
+                int n = ruleSets.length;
+                while (--n >= 0) {
+                   String currentName = ruleSets[n].getName();
+                   if (currentName.equals("%spellout-numbering") || currentName.equals("%digits-ordinal") || currentName.equals("%duration")) {
+                       defaultRuleSet = ruleSets[n];
+                       return;
+                   } 
+                }
+
+                n = ruleSets.length;
+                while (--n >= 0) {
+                    if (ruleSets[n].isPublic()) {
+                        defaultRuleSet = ruleSets[n];
+                        break;
+                    }
+                }
             }
         } else if (ruleSetName.startsWith("%%")) {
             throw new IllegalArgumentException("cannot use private rule set: " + ruleSetName);
@@ -1313,7 +1371,7 @@ public class RuleBasedNumberFormat extends NumberFormat {
                 // rules specified in the description, and create a _new_
                 // collator based on the combinaiton of those rules
                 RuleBasedCollator temp = (RuleBasedCollator)Collator.getInstance(locale);
-                String rules = temp.getRules() + lenientParseRules;
+                String rules = temp.getRules() + (lenientParseRules == null ? "" : lenientParseRules);
 
                 collator = new RuleBasedCollator(rules);
                 collator.setDecomposition(Collator.CANONICAL_DECOMPOSITION);
@@ -1472,11 +1530,29 @@ public class RuleBasedNumberFormat extends NumberFormat {
         // {dlf} Initialization of a fraction rule set requires the default rule
         // set to be known.  For purposes of initialization, this is always the
         // last public rule set, no matter what the localization data says.
+
+        // Set the default ruleset to the last public ruleset, unless one of the predefined
+        // ruleset names %spellout-numbering, %digits-ordinal, or %duration is found
+
+        boolean defaultNameFound = false;                  
+        int n = ruleSets.length;
         defaultRuleSet = ruleSets[ruleSets.length - 1];
-        for (int i = ruleSets.length - 1; i >= 0; --i) {
-            if (!ruleSets[i].getName().startsWith("%%")) {
-                defaultRuleSet = ruleSets[i];
+
+        while (--n >= 0) {
+            String currentName = ruleSets[n].getName();
+            if (currentName.equals("%spellout-numbering") || currentName.equals("%digits-ordinal") || currentName.equals("%duration")) {
+                defaultRuleSet = ruleSets[n];
+                defaultNameFound = true;                  
                 break;
+            } 
+        }
+
+        if ( !defaultNameFound ) {
+            for (int i = ruleSets.length - 1; i >= 0; --i) {
+                if (!ruleSets[i].getName().startsWith("%%")) {
+                    defaultRuleSet = ruleSets[i];
+                    break;
+                }
             }
         }
 
