@@ -1,5 +1,5 @@
 /*
- * (C) Copyright IBM Corp. 1998-2004.  All Rights Reserved.
+ * (C) Copyright IBM Corp. 1998-2007.  All Rights Reserved.
  *
  * The program is provided "as is" without any warranty express or
  * implied, including the warranty of non-infringement and the implied
@@ -10,6 +10,7 @@
  * IBM has been advised of the possibility of their occurrence. IBM
  * will not be liable for any third party claims against you.
  */
+
 package com.ibm.richtext.test;
 
 import java.util.Random;
@@ -90,11 +91,11 @@ public class TestTextPanel /*extends TestFmwk*/ {
             assertNotExpectingEvents(false, 0, false);
         }
         
-        void assertNotExpectingEvents(int iterationCount, boolean exp) {
-            assertNotExpectingEvents(true, iterationCount, exp);
+        void assertNotExpectingEvents(int iterCount, boolean exp) {
+            assertNotExpectingEvents(true, iterCount, exp);
         }
         
-        private void assertNotExpectingEvents(boolean logDetails, int iterationCount, boolean exp) {
+        private void assertNotExpectingEvents(boolean logDetails, int iterCount, boolean exp) {
             
             boolean e = false;
             for (int i=0; i < status.length; i++) {
@@ -102,7 +103,7 @@ public class TestTextPanel /*extends TestFmwk*/ {
                     if (logDetails) {
                         logMessage("Expecting event " +
                                         (i+TextPanelEvent.TEXT_PANEL_FIRST));
-                        logMessage("iterationCount="+iterationCount+";  expexting="+exp);
+                        logMessage("iterationCount="+iterCount+";  expexting="+exp);
                     }
                     e = true;
                 }
@@ -839,4 +840,4 @@ public class TestTextPanel /*extends TestFmwk*/ {
         }
         fListener.allowAll();
     }
-}
\ No newline at end of file
+}
